/*:
## **Aufgabe Ampel**
*/
/**
Beschreibt eine Lampe für eine Ampel

- author: Albert Wiedemann
- version: 1.0
 */
class Lampe
{
    /** Farbe der Lampe wenn eingeschaltet */
    var farbe: String
    /** Rahmenrechteck */
    var rahmen: Rechteck
    /** Hintergrund */
    var fläche: Rechteck
    /** Birne */
    var anzeige: Kreis
    
    /**
	Legt eine Lampe an der gegebenen Position an.
	- parameters:
		- x: x-Koordinate der Lampe
		- y: y-Koordinate der Lampe
		- farbe: Farbe der Lampe
     */
    init(x: Int, y: Int, farbe: String)
    {
        rahmen = Rechteck()
        rahmen.PositionSetzen(x: x, y: y)
        rahmen.GrößeSetzen(breite: 20, höhe: 20)
        rahmen.FarbeSetzen(farbe: "schwarz")
        fläche = Rechteck()
        fläche.PositionSetzen(x: x + 1, y: y + 1)
        fläche.GrößeSetzen(breite: 18, höhe: 18)
        fläche.FarbeSetzen(farbe: "weiß")
        anzeige = Kreis()
        anzeige.RadiusSetzen(radius: 5)
        anzeige.PositionSetzen(x: x + 10, y: y + 10)
        anzeige.FarbeSetzen(farbe: farbe)
        self.farbe = farbe
    }
    
    /**
    Schaltet die Lampe ein
    */
    func Einschalten()
    {
        anzeige.FarbeSetzen(farbe: farbe)
    }
    
    /**
    Schaltet die Lampe aus
    */
    func Ausschalten()
    {
        anzeige.FarbeSetzen(farbe: "schwarz")
    }
    
    /**
    Setzt die Position der Lampe
	- parameters:
    	- x: x-Koordinate der Lampe
     	- y:  y-Koordinate der Lampe
    */
    func PositionSetzen(x: Int, y: Int)
    {
        rahmen.PositionSetzen(x: x, y: y)
        fläche.PositionSetzen(x: x + 1, y: y + 1)
        anzeige.PositionSetzen(x: x + 10, y: y + 10)
    }
}

/**
Eine Ampel verwendet zur Anzeige drei Lampen.
Die Ampel verwaltet ihre Position als Ganzes, ihre Ausrichtung und auch die
nach den Verkehrsregeln möglichen Ampelphasen.

- author: Albert Wiedemann
- version: 1.0
 */
class Ampel
{
    /** Mögliche Werte sind die in Deutschland erlaubten Ampelphasen grün, gelb, rot und rotgelb. */
    var ampelphase: String
    /** obere Lampe (rot) */
    var lampeOben: Lampe
    /** mittlere Lampe (gelb) */
    var lampeMitte: Lampe
    /** untere Lampe (grün) */
    //Ergänzen

    /**
    Konstruktor für Objekte der Klasse Ampel.
    */
    convenience init()
    {
        self.init(x: 100, y: 100)
    }
   
    /**
    Konstruktor für Objekte der Klasse Ampel
    mit folgenden Eingabewerten.
	- parameters:
        - x: x-Position
        - y:  y-Position
    */
    init(x: Int, y: Int)
    {
        ampelphase = "rot"
        lampeOben = Lampe(x: x, y: y, farbe: "rot")
        lampeMitte = Lampe(x: x, y: y + 20, farbe: "gelb")
        //Ergänzen
       
        lampeOben.Einschalten()
        lampeMitte.Ausschalten()
        //Ergänzen
    }

    /**
    Setzt die Position der Ampel.
	- parameters:
        - x: x-Position
        - y:  y-Position
    */
    func PositionSetzen(x: Int, y: Int)
    {
        if x<=800 && x>=0 && y<=600 && y>=0
        {
            lampeOben.PositionSetzen(x: x, y: y)
            lampeMitte.PositionSetzen(x: x, y: y + 20)
            //Ergänzen
        }
    }
           
    /**
    Setzt die Ampel auf grün.
    */
    func GrünSetzen()
    {
        ampelphase = "grün"
        lampeOben.Ausschalten()
        lampeMitte.Ausschalten()
        //Ergänzen
    }
}

let a = Ampel()
