/*:
## **Aufgabe Countdown und Uhr – Zeitanzeige in Teile zerlegen**
*/

/**
Nummernanzeige mit zwei Ziffern
*/
internal class Nummernanzeige
{
    private var obergrenze: Int
    private var wert: Int

    /**
    Der Konstruktor belegt die Obergrenze und setzt den Startwert
	- parameters:
		- obergrenzeNeu: die gewünschte Obergrenze
     */
    internal init(obergrenzeNeu: Int)
    {
        obergrenze = obergrenzeNeu
        wert = 0
    }

    /**
    Gibt die eingestellte Obergrenze zurück
	- returns: eingestellte Obergrenze
    */
    internal func ObergrenzeGeben() -> Int
    {
        return obergrenze
    }

    /**
    Gibt den aktuellen Wert zurück
	- returns: aktueller Wert
    */
    internal func WertGeben() -> Int
    {
        return wert
    }

    /**
    Setzt den Wert der Anzeige
	- parameters:
		- wertNeu: neuer Wert für die Anzeige
    */
    internal func WertSetzen(wertNeu: Int)
    {
        if wertNeu >= 0 && wertNeu < obergrenze
        {
            wert = wertNeu
        }
    }

    /**
	Erhöht den Wert der Anzeige (um 1)
	
	Bei Erreichen der Obergrenze wird auf 0 zurück gesetzt.
     */
    internal func Erhöhen()
    {
        wert = wert + 1
        if wert == obergrenze
        {
            wert = 0
        }
    }

    /**
	Erniedrigt den Wert der Anzeige (um 1)
	
	Bei Erreichen der 0 wird auf die Obergrenze zurück gesetzt.
     */
    internal func Erniedrigen()
    {
        wert = wert - 1
        if (wert < 0)
        {
            wert = obergrenze-1
        }
    }
}

/**
Digitaluhr
*/
internal class Uhr: Ereignisbehandlung
{
    private var minuten: Nummernanzeige
    private var sekunden: Nummernanzeige
    
    private var anzeige: Text

    /**
    Konstruktor
    */
    internal override init()
    {
        minuten = Nummernanzeige(obergrenzeNeu: 60)
        sekunden = Nummernanzeige(obergrenzeNeu: 60)
        
        //Darstellung
        anzeige = Text()
        anzeige.PositionSetzen(x: 100,y: 100)
        anzeige.TextGrößeSetzen(größe: 40)
        super.init()
        anzeige.TextSetzen(text: AnzeigeAusgeben())
    }

    /**
	Erstellt aus den Zeitbestandteilen eine Anzeige als Zeichenkette
	- returns:Anzeige als Zeichenkette
     */
    internal func AnzeigeAusgeben() -> String
    {
        var gesamtAnzeige: String
        gesamtAnzeige = ZweistelligeZahlAlsTextErstellen(zahl: minuten.WertGeben()) + ":" + ZweistelligeZahlAlsTextErstellen(zahl: sekunden.WertGeben())
        return gesamtAnzeige
    }

    /**
    Wandelt eine ein- oder zweistellige Zahl in einen String mit 2 Zeichen um
	- parameters:
		- zahl: die umzuwandelnde Zahl
	- returns: die zugehörige Zeichenkette
     */
    internal func ZweistelligeZahlAlsTextErstellen(zahl: Int) -> String
    {
        if (zahl < 10)
        {
            return "0\(zahl)"
        }
        else {
            return "\(zahl)"
        }
    }

    /**
	Schaltet die Uhr um eine Sekunde weiter
     */
    internal override func TaktImpulsAusführen()
    {
        sekunden.Erhöhen()
        if sekunden.WertGeben() == 0
        {
            minuten.Erhöhen()
        }
        anzeige.TextSetzen(text: AnzeigeAusgeben())
    }
}

Uhr()
