/*:
## **Aufgabe Grafische Benutzeroberflächen, selbst gemacht! **
*/


/**
Klasse zur Beschreibung eines interaktiven Textfeldes

- author: Klaus Reinold
- version: 1.0
*/
class Textfeld: Ereignisbehandlung
{
    /** x-Position */
    var x: Int
    /** y-Position */
    var y: Int
    /** Breite des Textfelds */
    var breite: Int
    /** Inhalt des Textfelds */
    var inhalt: String
    /** Gibt an, ob das Textfeld gerade aktiv ist. */
    var istAngeklickt: Bool
    /** Das Rechteck als Hintergrund */
    var rahmen: Rechteck
    /** Das darstellende Textfeld */
    var text: Text

    /**
    Konstruktor des Textfeldes
    */
    override init()
    {
        x = 10
        y = 10
        breite = 300
        inhalt = "Klick mich an und schreib was rein."
        istAngeklickt = false

        rahmen = Rechteck()
        rahmen.GrößeSetzen(breite: 300, höhe: 20)
        rahmen.PositionSetzen(x: x, y: y)
        rahmen.FarbeSetzen(farbe: "weiß")

        text = Text()
        text.PositionSetzen(x: x + 5, y: y + 12)
        text.TextSetzen(text: inhalt)

		super.init()
    }

    /**
    Ereignisroutine des Mausklicks
	- parameters:
        - x: x-Position des Mausklicks
        - y: y-Position des Mausklicks
        - anzahl: Anzahl der Klicks
    */
    override func MausGeklickt(x: Int, y: Int, anzahl: Int)
    {
        if x >= self.x && x <= self.x + breite && y > self.y && y < self.y + 20
        {
            istAngeklickt = true
        }
        else
        {
            istAngeklickt = false
        }
    }

    /**
    Ereignisroutine des Tastendrucks
	- parameters:
		- taste: gedrückte Taste
    */
    override func TasteGedrückt(taste: Character)
    {
        if istAngeklickt
        {
            if inhalt == "Klick mich an und schreib was rein."
            {
                inhalt = String(taste)
            }
            else
            {
                inhalt = inhalt + String(taste)
            }
            text.TextSetzen(text: inhalt)
        }
    }

    /**
	Position des Textfeldes setzen
	- parameters:
		- xNeu: neue x-Koordinate
		- yNeu: neue y-Koordinate
    */
    func PositionSetzen(xNeu: Int, yNeu: Int)
    {
        x = xNeu
        y = yNeu
        rahmen.PositionSetzen(x: x, y: y)
        text.PositionSetzen(x: x + 5, y: y + 12)
    }
    
    /**
	Methode zum Setzen des Textes
	- parameters:
		- tNeu: neuer Text
    */
    func TextSetzen(tNeu: String)
    {
        inhalt = tNeu
        text.TextSetzen(text: inhalt)
    }
}

/**
Klasse zur Beschreibung eines Buttons

- author: Klaus Reinold
- version: 1.0
*/
internal class Button: Ereignisbehandlung
{
    /** x-Position */
    private var x: Int
    /** y-Position */
    private var y: Int
    /** Breite des Button */
    private var breite: Int
    /** Rechteck zur Darstellung des Buttons */
    private var rahmen: Rechteck
    /** Textfeld zur Beschriftung des Buttons */
    private var text: Text
    /** Farbe des Buttons */
    private var farbe: String


    /**
    Konstruktor des Buttons
    */
    internal override init()
    {
        x = 50
        y = 50
        breite = 70

        farbe = "rot"
        rahmen = Rechteck()
        rahmen.GrößeSetzen(breite: 70, höhe: 20)
        rahmen.PositionSetzen(x: x, y: y)
        rahmen.FarbeSetzen(farbe: farbe)

        text = Text();
        text.PositionSetzen(x: x + 5, y: y + 12)
        text.TextSetzen(text: "Klick mich")

		super.init()
    }

    /**
    Ereignisroutine des Mausklicks
	- parameters:
        - x: x-Position des Mausklicks
        - y: y-Position des Mausklicks
        - anzahl: Anzahl der Klicks
    */
    override internal func MausGeklickt(x: Int, y: Int, anzahl: Int)
    {
        if x >= self.x && x <= self.x + breite && y > self.y && y < self.y + 20
        {
            AufMausklickReagieren()
        }
    }

    /**
    Farbe des Buttons setzen
	- parameters:
		- farbeNeu: Farbe
    */
    internal func FarbeSetzen(farbeNeu: String)
    {
        rahmen.FarbeSetzen(farbe: farbeNeu)
    }

    /**
    Beschriftung des Buttons setzen
     * @param textNeu Text
    */
    internal func TextSetzen(textNeu: String)
    {
        text.TextSetzen(text: textNeu)
    }

    /**
    Position des Buttons setzen
	- parameters:
		- xNeu: neue x-Koordinate
		- yNeu: neue y-Koordinate
    */
    internal func PositionSetzen(xNeu: Int, yNeu: Int)
    {
        x = xNeu
        y = yNeu
        rahmen.PositionSetzen(x: x, y: y)
        text.PositionSetzen(x: x + 5, y: y + 12)
    }

    /**
    Methode definiert die Reaktion, falls der Button angeklickt wurde.
    Methode bei Bedarf überschreiben!
    */
    fileprivate func AufMausklickReagieren()
    {
        if farbe == "rot"
        {
            farbe = "grün"
        }
        else
        {
            farbe = "rot"
        }
        FarbeSetzen(farbeNeu: farbe)
    }
}

/**
Beschreiben Sie hier die Klasse GUI.

- author: Klaus Reinold
- version: 0.9
 */
internal class GUI
{
    /** Das Textfeld */
    private var t: Textfeld
    /** Der Button */
    private var b: Button

    /**
    Ergänze hier eine passende Dokumentation der Klasse MeinButton
    */
    class MeinButton: Button
    {
    	/** Das zu löschende Textfeld */
		private var t: Textfeld
        
    	/**
        Der Kontruktor übergibt das Textfeld
         */
        init(t: Textfeld)
        {
        	self.t = t
        }
        
        /**
        Ergänze hier eine passende Dokumentation der Methode
        */
        override fileprivate func AufMausklickReagieren()
        {
            t.TextSetzen(tNeu: "")
        }
        
    }
    /**
    Konstruktor der GUI
    */
    internal init()
    {
        t = Textfeld()
        b = MeinButton(t: t)
    }
}

GUI()

