/*:
## **Aufgabe Verkehrssimulation (Teil 1)**
*/
//#-editable-code

/**
Beschreibt eine Lampe für eine Ampel

- author: Albert Wiedemann
- version: 1.0
 */
class Lampe
{
    /** Farbe der Lampe wenn eingeschaltet */
    var farbe: String
    //Referenzattribute
    var rahmen: Rechteck
    var fläche: Rechteck
    var anzeige: Kreis
    
    /**
     * Legt eine Lampe an der gegebenen Position an.
     * @param x x-Koordinate der Lampe
     * @param y y-Koordinate der Lampe
     * @param farbe Farbe der Lampe
     */
    init(x: Int, y: Int, farbe: String)
    {
        rahmen = Rechteck()
        rahmen.PositionSetzen(x: x, y: y)
        rahmen.GrößeSetzen(breite: 20, höhe: 20)
        rahmen.FarbeSetzen(farbe: "schwarz")
        fläche = Rechteck()
        fläche.PositionSetzen(x: x + 1, y: y + 1)
        fläche.GrößeSetzen(breite: 18, höhe: 18)
        fläche.FarbeSetzen(farbe: "weiß")
        anzeige = Kreis()
        anzeige.RadiusSetzen(radius: 5)
        anzeige.PositionSetzen(x: x + 10, y: y + 10)
        anzeige.FarbeSetzen(farbe: farbe)
        self.farbe = farbe
    }
    
    /**
    Schaltet die Lampe ein
    */
    func Einschalten()
    {
        anzeige.FarbeSetzen(farbe: farbe)
    }
    
    /**
    Schaltet die Lampe aus
    */
    func Ausschalten()
    {
        anzeige.FarbeSetzen(farbe: "schwarz")
    }
    
    /**
    Setzt die Position der Lampe
	- parameters:
    	- x: x-Koordinate der Lampe
     	- y:  y-Koordinate der Lampe
    */
    func PositionSetzen(x: Int, y: Int)
    {
        rahmen.PositionSetzen(x: x, y: y)
        fläche.PositionSetzen(x: x + 1, y: y + 1)
        anzeige.PositionSetzen(x: x + 10, y: y + 10)
    }
}

/**
Eine Ampel verwendet zur Anzeige drei Lampen.
Die Ampel verwaltet ihre Position als Ganzes, ihre Ausrichtung und auch die
nach den Verkehrsregeln möglichen Ampelphasen.

- author: Albert Wiedemann
- version: 1.0
 */
class Ampel: Ereignisbehandlung
{
    /** Mögliche Werte sind die in Deutschland erlaubten Ampelphasen grün, gelb, rot und rotgelb. */
    var ampelphase: String
    /** Wartezeit bis zum Weiterschalten */
    var wartezeit: Int
    
    // Referenzattribute
    var lampeOben: Lampe
    var lampeMitte: Lampe
    var lampeUnten: Lampe

   
    /**
    Konstruktor für Objekte der Klasse Ampel.
    */
    convenience override init()
    {
        self.init(x: 100, y: 100)
    }
   
    /**
    Konstruktor für Objekte der Klasse Ampel
    mit folgenden Eingabewerten.
	- parameters:
        - x: x-Position
        - y:  y-Position
    */
    init(x: Int, y: Int)
    {
        ampelphase = "rot"
        wartezeit = 0
        lampeOben = Lampe(x: x, y: y, farbe: "rot")
        lampeMitte = Lampe(x: x, y: y + 20, farbe: "gelb")
        lampeUnten = Lampe(x: x, y: y + 40, farbe: "grün")
        super.init()
       
        lampeOben.Einschalten()
        lampeMitte.Ausschalten()
        lampeUnten.Ausschalten()
    }
           
    /**
    Setzt die Ampel auf grün.
    */
    func GrünSetzen()
    {
        ampelphase = "grün"
        lampeOben.Ausschalten()
        lampeMitte.Ausschalten()
        lampeUnten.Einschalten()
    }
    
    /**
    Setzt die Ampel auf gelb.
    */
    func GelbSetzen()
    {
        ampelphase = "gelb"
        lampeOben.Ausschalten()
        lampeMitte.Einschalten()
        lampeUnten.Ausschalten()
    }
    
    /**
    Setzt die Ampel auf rot-gelb.
    */
    func RotgelbSetzen()
    {
        ampelphase = "rotgelb"
        lampeOben.Einschalten()
        lampeMitte.Einschalten()
        lampeUnten.Ausschalten()
    }
    
    /**
    Setzt die Ampel auf rot.
    */
    func RotSetzen()
    {
        ampelphase = "rot"
        lampeOben.Einschalten()
        lampeMitte.Ausschalten()
        lampeUnten.Ausschalten()
    }
    
    /**
    Schaltet die Ampelphasen in der Reihenfolge rot,
    rotgelb, grün, gelb
    */
    func Weiterschalten()
    {
        if ampelphase == "rot"
        {
            RotgelbSetzen()
        }
        else
        {
            if ampelphase == "rotgelb"
            {
                GrünSetzen()
            }
            else
            {
                if ampelphase == "grün"
                {
                    GelbSetzen()
                }
                else
                {
                     RotSetzen()
                }
            }
        }
    }

    /**
    Schaltet zur gegebenen Zeit die Ampel weiter
    */
    override func TaktImpulsAusführen()
    {
        if wartezeit <= 0
        {
            Weiterschalten()
            if (ampelphase == "rot") || (ampelphase == "grün")
            {
                wartezeit = 30
            }
            else
            {
                wartezeit = 5
            }
        }
        else
        {
            wartezeit = wartezeit - 1
        }
    }
    
    /**
    Setzt die Position der Ampel.
	- parameters:
        - x: x-Position
        - y:  y-Position
    */
    func PositionSetzen(x: Int, y: Int)
    {
        if x<=800 && x>=0 && y<=600 && y>=0
        {
            lampeOben.PositionSetzen(x: x, y: y)
            lampeMitte.PositionSetzen(x: x, y: y + 20)
            lampeUnten.PositionSetzen(x: x, y: y + 40)
        }
    }

    /**
    Meldet, ob die Ampel auf grün steht.
    - returns: true, falls die Ampel grün zeigt.
    */
    func IstGrün() -> Bool
    {
        return ampelphase == "grün"
    }

    /**
    Meldet, ob die Ampel auf gelb steht.
    - returns: true, falls die Ampel gelb zeigt.
    */
    func IstGelb() -> Bool
    {
        return ampelphase == "gelb"
    }
}

/**
Ein Fahrzeugsymbol

- author: Albert Wiedemann
- version: 1.0
*/
class FahrzeugSymbol
{
    /** Anzeigegröße für ein Fahrzeug */
    var länge: Int
    var breite: Int

    /** Interna */
    /** Farbe des Fahrzeugs */
    var aktFarbe: String
    /** x-Position des Fahrzeugs */
    var x: Int
    /** y-Position des Fahrzeugs */
    var y: Int
    /** Fahrzeugaufbau */
    let aufbau: Rechteck
    /** Frontscheibe */
    let scheibe: Rechteck

    /**
    Standardkonstruktor für Objekte der Klasse FahrzeugSymbol.
    */
    init()
    {
        aktFarbe = "blau"
        breite = 9
        länge = 22
        x = 0
        y = 0
        aufbau = Rechteck()
        scheibe = Rechteck()
        aufbau.PositionSetzen(x: 0, y: 0)
        aufbau.GrößeSetzen(breite: breite, höhe: länge)
        aufbau.FarbeSetzen(farbe: aktFarbe)
        scheibe.PositionSetzen(x: 1, y: länge/4)
        scheibe.GrößeSetzen(breite: breite - 2, höhe: länge/4)
        scheibe.FarbeSetzen(farbe: "weiß")
    }

    /**
	Setzt die Farbe des Fahrzeugsymbols.
	- parameters:
		- neueFarbe: die neue Farbe des Fahrzeugsymbols
    */
    func FarbeSetzen(neueFarbe: String)
    {
        aktFarbe = neueFarbe;
        aufbau.FarbeSetzen(farbe: aktFarbe);
    }

    /**
    Setzt die Position des Fahrzeugsymbols.
    Die Koordinaten bezeichnen die linke, obere Ecke des Fahrzeugsymbols.
	- parameters:
        - x: x-Position
        - y:  y-Position
    */
    func PositionSetzen(x: Int, y: Int)
    {
        self.x = x
        self.y = y
        aufbau.PositionSetzen(x: x, y: y)
        scheibe.PositionSetzen(x: x + 1, y: y + länge/4)
    }

    /**
    Entfernt das Fahrzeugsymbol aus der Anzeige
    */
    func Entfernen()
    {
        aufbau.Entfernen()
        scheibe.Entfernen()
    }
}

/**
Beschreibt ein Fahrzeug.

- author: Albert Wiedemann
- version: 1.0
*/
class Fahrzeug
{
    /** Anzeigesymbol */
    var symbol: FahrzeugSymbol
    /** Bewegungsdaten */
    var x: Int
    var y: Int
    var v: Double
    var vmax: Double
    let a: Double

    /**
    Erzeugt die Anzeige und setzt die Startdaten
    */
    init()
    {
        vmax = 13.89
        v = vmax
        x = 0
        y = 0
        a = 4.0
        symbol = FahrzeugSymbol()
        symbol.FarbeSetzen(neueFarbe: ZufallsfarbeErzeugen())
        symbol.PositionSetzen(x: x, y: y)
    }

    /**
    Erzeugt eine zufällige Farbe für das Fahrzeug
    */
    func ZufallsfarbeErzeugen() -> String
    {
        switch Zufall().ZufallGanz(top: 7)
        {
            case 0:
                return "rot"
            case 1:
                return "grün"
            case 2:
                return "blau"
            case 3:
                return "gelb"
            case 4:
                return "cyan"
            case 5:
                return "magenta"
            case 6:
                return "weiss"
            default:
                return ""
        }
    }
    
    /**
    Setzt die Position des Fahrzeugs.
	- parameters:
        - x: x-Position
        - y:  y-Position
    */
    func PositionSetzen(x: Int, y: Int)
    {
        self.x = x
        self.y = y
        symbol.PositionSetzen(x: x, y: y)
    }
    
    /**
    Fährt das Fahrzeug eine Sekunde nach oben.
	- parameters:
		- wegmax: maximaler Weg bis zum nächsten Hindernis (Ampel oder Vorgänger)
		- vvorg: Geschwindigkeit des Vorgängerfahrzeugs bzw.der Ampel
     */
    func Fahren(wegmax: Int, vvorg: Double)
    {
        var weg: Double
        var vneu: Double
        weg = 0
        for _ in 1 ... 10            // Maximaler Weg
        {
            vneu = a * 0.1 + v
            if vneu > vmax
            {
                vneu = vmax
            }
            weg = weg + (vneu + v) / 2.0 * 0.1
            v = vneu;
        }
        weg = weg * 5;                        // 1Px = 20cm
        if wegmax <= 0                     // Muss stehen bleiben
        {
            weg = 0.0
            v = 0.0
        }
        else if weg >= Double(wegmax)                 // Kommt zu dicht an den Vorgänger
        {
            weg = Double(wegmax)
            if v > vvorg                      // und ist auch noch zu schnell
            {
                v = vvorg
            }
        }
        else
        {
            vneu = (Double(wegmax) / 5.0 + vvorg * 1.0 - weg / 5.0) / 1.8 // Maximale Geschwindigkeit für den Restabstand zum Vorgänger
            if vneu < v                       // Zu schnell für den Abstand
            {
                v = vneu
            }
        }
        PositionSetzen(x: x, y: y - Int(weg))
    }

    /**
    Meldet die x-Koordinate zurück
    - returns: x-Koordinate
    */
    func XKoordinateGeben() -> Int
    {
        return x
    }

    /**
    Meldet die y-Koordinate zurück
    - returns: y-Koordinate
    */
    func YKoordinateGeben() -> Int
    {
        return y
    }

    /**
    Meldet die Länge des Fahrzeugs zurück
    - returns: Länge
    */
    func LängeGeben() -> Int
    {
        return 22
    }

    /**
    Meldet die Geschwindigkeit zurück
    - returns: Geschwindigkeit
    */
    func GeschwindigkeitGeben() -> Double
    {
        return v
    }

    /**
    Entfernt das Fahrzeug aus der Anzeige
    */
    func Entfernen()
    {
        symbol.Entfernen()
    }
}

/**
Stellt eine Fahrbahn dar

- author: Albert Wiedemann
- version: 1.0
*/
class FahrbahnSymbol
{
    /** x-Position der linken, oberen Ecke */
    var x: Int
    /** y-Position der linken, oberen Ecke */
    var y: Int
    /** Fahrbahnbelag */
    var hintergrund: Rechteck
    /** Haltelinie */
    var haltelinie: Rechteck

    /**
    Erzeugt das FahrbahnSymbol
    */
    init()
    {
        x = 0
        y = 0
        hintergrund = Rechteck()
        hintergrund.PositionSetzen(x: x, y: y)
        hintergrund.GrößeSetzen(breite: 20, höhe: 600)
        hintergrund.FarbeSetzen(farbe: "grau")
        haltelinie = Rechteck()
        haltelinie.PositionSetzen(x: x + 3, y: y + 200)
        haltelinie.GrößeSetzen(breite: 17, höhe: 3)
        haltelinie.FarbeSetzen(farbe: "weiß")
    }

    /**
    Setzt die Position der Fahrbahn.
	- parameters:
    	- x: x-Position des Kreuzungsmittelpunkts
        - y: y-Position des Kreuzungsmittelpunkts
     */
    func PositionSetzen(x: Int, y: Int)
    {
        self.x = x
        self.y = y
        hintergrund.PositionSetzen(x: x, y: y)
        haltelinie.PositionSetzen(x: x + 3, y: y + 200)
    }

    /**
    Meldet die Eintrittsposition des Fahrzeugs
    - returns: Startposition des Fahrzeugs
    */
    func EintrittsPositionGeben() -> Int
    {
        return y + 600
    }

    /**
    Meldet die Austrittsposition des Fahrzeugs
    - returns: Austrittsposition des Fahrzeugs
    */
    func AustrittsPositionGeben() -> Int
    {
        return y
    }

    /**
    Meldet den Abstand zum inneren Fahrbahnrand
    - returns: Abstand des Fahrzeugs
    */
    func RandAbstandGeben() -> Int
    {
        return 5
    }

    /**
    Meldet die Position der Haltelinie
    - returns: Haltelinienposition
    */
    func HaltelinienPositionGeben() -> Int
    {
        return y + 205
    }
}

/**
Verwaltet die Fahrzeuge auf einer Fahrbahn.
 
- author: Albert Wiedemann
- version: 1.0
*/
class Fahrbahn: Ereignisbehandlung
{
    /** x-Position */
    var x: Int
    /** y-Position */
    var y: Int
    /** start-Position */
    var start: Int
    /** ende-Position */
    var ende: Int
    /** rand-Position */
    var rand: Int
    /** Wartezeit zur nächsten Fahrzeugerzeugung */
    var wartezeit: Int
    
    /** erstes Fahrzeug */
    var fahrzeug1: Fahrzeug?
    /** zweites Fahrzeug */
    var fahrzeug2: Fahrzeug?
    /** Darstellungssymbol */
    let darstellung: FahrbahnSymbol
    /** zuständige Ampel */
    let ampel: Ampel

    /**
    Erzeugt eine leere Fahrbahn ab dem linken Rand unterhalb der Fenstermitte.
	- parameters:
    	- a: für diese Richtung zuständige Ampel
    	- x: x-Koordinate der Fahrbahn
    	- y: y-Koordinate der Fahrbahn
    */
    init(a: Ampel, x: Int, y: Int)
    {
        self.x = x
        self.y = y
        ampel = a
        fahrzeug1 = nil
        fahrzeug2 = nil
        darstellung = FahrbahnSymbol()
        darstellung.PositionSetzen(x: x, y: y)
        start = darstellung.EintrittsPositionGeben()
        ende = darstellung.AustrittsPositionGeben()
        rand = darstellung.RandAbstandGeben()
        wartezeit = 0
        super.init()
    }

    /**
    Reiht ein neues Fahrzeug an der Startposition in die Fahrbahn ein.
	- parameters:
    	- f: das neue Fahrzeug
     */
    func FahrzeugEinreihen(f: Fahrzeug)
    {
        if fahrzeug1 == nil
        {
            fahrzeug1 = f
        }
        else if fahrzeug2 == nil
        {
            fahrzeug2 = f
        }
        f.PositionSetzen(x: x + rand, y: start)
    }

    /**
    Ermittelt, ob ein Fahrzeug entfernt werden muss
	- parameters:
    	- f: das in Frage kommende Fahrzeug
	- returns: true, wenn das Fahrzeug die Fahrbahn verlassen hat
    */
    func IstDraussen(f: Fahrzeug) -> Bool
    {
        return f.YKoordinateGeben() < ende
    }

    /**
    Ermittelt, ob das Fahrzeug sich nach dem Vorgänger richten oder
    die Ampel beachten muss.
	- parameters:
    	- ampelabstand: Abstand zur Ampel (Haltelinie)
    	- vorgängerabstand: Abstand zum Vorgängerfahrzeug
    	- f: das aktuelle Fahrzeug
    - returns: true, wenn der Vorgänger wichtig ist, false, wenn die Ampel zählt
    */
    func VorgängerZählt(ampelabstand: Int, vorgängerabstand: Int, f: Fahrzeug) -> Bool
    {
        return (ampelabstand < 0) || (vorgängerabstand < ampelabstand) || ampel.IstGrün() ||
               ampel.IstGelb() && (Double(ampelabstand) < f.GeschwindigkeitGeben() * 1.8 * 5) && (f.GeschwindigkeitGeben() > 3.0)
    }

    /**
    Bewegt alle Fahrzeuge der Fahrbahn für eine Sekunde.
    */
    func FahrzeugeBewegen()
    {
        var vvorgänger:Double
        var wegmax: Int
        var ampelAbstand: Int
        
        //1. Fahrzeug
        if fahrzeug1 != nil
        {
            wegmax = 100000
            vvorgänger = 100
            ampelAbstand = fahrzeug1!.YKoordinateGeben() - darstellung.HaltelinienPositionGeben()
            if VorgängerZählt(ampelabstand: ampelAbstand, vorgängerabstand: wegmax, f: fahrzeug1!)
            {
                fahrzeug1!.Fahren(wegmax: wegmax, vvorg: vvorgänger)
            }
            else
            {
                fahrzeug1!.Fahren(wegmax: ampelAbstand, vvorg: 0.0)
            }
            //2. Fahrzeug
            if fahrzeug2 != nil
            {
                vvorgänger = fahrzeug1!.GeschwindigkeitGeben()
                wegmax = fahrzeug2!.YKoordinateGeben() - fahrzeug1!.YKoordinateGeben() - fahrzeug1!.LängeGeben() - 2
                if vvorgänger > 0
                {
                    wegmax = wegmax - Int(((fahrzeug2!.GeschwindigkeitGeben() + vvorgänger) / 2.0 * 1.8 * 5).rounded())
                }
                ampelAbstand = fahrzeug2!.YKoordinateGeben() - darstellung.HaltelinienPositionGeben()
                if VorgängerZählt(ampelabstand: ampelAbstand, vorgängerabstand: wegmax, f: fahrzeug2!)
                {
                    fahrzeug2!.Fahren(wegmax: wegmax, vvorg: vvorgänger)
                }
                else
                {
                    fahrzeug2!.Fahren(wegmax: ampelAbstand, vvorg: 0.0)
                }
            }
        }
        if (fahrzeug1 != nil) && IstDraussen(f: fahrzeug1!)
        {
            fahrzeug1!.Entfernen()
            fahrzeug1 = fahrzeug2
            fahrzeug2 = nil
        }
    }
    
    /**
    Setzt die Position der Fahrbahn.
	- parameters:
        - x: x-Position
    	- y: y-Position
    */
    func PositionSetzen(x: Int, y: Int)
    {
        self.x = x
        self.y = y
        darstellung.PositionSetzen(x: x, y: y)
        rand = darstellung.RandAbstandGeben()
    }
    
    /**
    Ausgabe der Fahrzeuganzahl
    - returns: Anzahl der Fahrzeuge auf der Fahrbahn
    */
    func FahrzeuganzahlGeben() -> Int
    {
        return fahrzeug1 == nil ? 0 : fahrzeug2 == nil ? 1 : 2
    }

    /**
    Fährt die Fahrzeuge und erzeugt gegebenenfalls ein neues Fahrzeug.
    */
    override func TaktImpulsAusführen()
    {
        FahrzeugeBewegen()
        if wartezeit <= 0
        {
            FahrzeugEinreihen(f: Fahrzeug())
            wartezeit = 4
        }
        else
        {
            wartezeit = wartezeit - 1
        }
    }
}


//#-end-editable-code

/**
Steuert die Simulation

- author: Albert Wiedemann
- version: 1.0
*/
class Simulation: Ereignisbehandlung
{
    /**
     * Legt die Objekte der Simulation an und startet den Taktgeber
     */
    override init()
    {
        super.init()
        Fahrbahn(a: Ampel(x: 350, y: 200), x: 300, y: 0)
        Starten()
    }
    
    /**
    Nichts, schaltet die Meldung aus.
    */
    override func TaktImpulsAusführen()
    {
    }
}


//#-editable-code
Simulation()
//#-end-editable-code
